/*
 * Copyright (c) 2022 - 2023, Nordic Semiconductor ASA
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <nrfx_example.h>
#include <nrfx_spim.h>
#include <nrfx_spis.h>

#define NRFX_LOG_MODULE                 EXAMPLE
#define NRFX_EXAMPLE_CONFIG_LOG_ENABLED 1
#define NRFX_EXAMPLE_CONFIG_LOG_LEVEL   3
#include <nrfx_log.h>

/**
 * @defgroup nrfx_spim_spis_non_blocking Non-blocking SPIM with SPIS example
 * @{
 * @ingroup nrfx_spim_spis_examples
 *
 * @brief Example showing the basic functionality of nrfx_spim and nrfx_spis drivers operating in the non-blocking mode.
 *
 * @details Application initializes nrfx_spim and nrfx_spis drivers. Then starts operating in the non-blocking mode.
 *          Message @ref MSG_TO_SEND_MASTER from @ref m_tx_buffer_master is transmitted to @ref m_rx_buffer_slave.
 *          Simultaneously message @ref MSG_TO_SEND_SLAVE from @ref m_tx_buffer_slave is transmitted to @ref m_rx_buffer_master.
 *          When the transactions are completed @ref spim_handler() and @ref spis_handler() are executed.
 *          Then received messages are read from @ref m_rx_buffer_master and @ref m_rx_buffer_slave.
 */

/** @brief Symbol specifying SPIS instance to be used. */
#define SPIS_INST_IDX 2

#define MOSI_PIN_SLAVE  26
#define MISO_PIN_SLAVE  25
#define SCK_PIN_SLAVE   7
#define CSN_PIN_SLAVE   35

/** @brief Symbol specifying message to be sent via SPIM data transfer. */
#define MSG_TO_SEND_MASTER "Semiconductor"

/** @brief Symbol specifying message to be sent via SPIS data transfer. */
#define MSG_TO_SEND_SLAVE "Nordic"

/** @brief Transmit buffer initialized with the specified message ( @ref MSG_TO_SEND_SLAVE ). */
static uint8_t m_tx_buffer_slave[] = MSG_TO_SEND_SLAVE;

/** @brief Receive buffer defined with the size to store specified message ( @ref MSG_TO_SEND_SLAVE ). */
static uint8_t m_rx_buffer_master[sizeof(MSG_TO_SEND_SLAVE)];

/** @brief Receive buffer defined with the size to store specified message ( @ref MSG_TO_SEND_MASTER ). */
static uint8_t m_rx_buffer_slave[sizeof(MSG_TO_SEND_MASTER)];

nrfx_spis_t spis_inst = NRFX_SPIS_INSTANCE(SPIS_INST_IDX);

/**
 * @brief Function for handling SPIS driver events.
 *
 * @param[in] p_event   Pointer to the SPIS driver event.
 * @param[in] p_context Pointer to the context passed from the driver.
 */
static void spis_handler(nrfx_spis_evt_t const * p_event, void * p_context)
{
    if (p_event->evt_type == NRFX_SPIS_XFER_DONE)
    {
        printk("SPIS rx buffer:");
        for (uint8_t i = 0; i < sizeof(m_rx_buffer_slave); i ++) {
            printk("%02x,", m_rx_buffer_slave[i]);
        }
        printk("\r\n");

        memset(m_rx_buffer_slave, 0x00, sizeof(m_rx_buffer_slave));

        nrfx_err_t status;
        (void)status;
        status = nrfx_spis_buffers_set(&spis_inst,
                                    m_tx_buffer_slave, sizeof(m_tx_buffer_slave),
                                    m_rx_buffer_slave, sizeof(m_rx_buffer_slave));
    }
}

/**
 * @brief Function for application main entry.
 *
 * @return Nothing.
 */
int main(void)
{
    nrfx_err_t status;
    (void)status;
    void * p_context = "Some context";

    NRFX_EXAMPLE_LOG_INIT();

    NRFX_LOG_INFO("Starting nrfx_spim_spis basic non-blocking example.");
    NRFX_EXAMPLE_LOG_PROCESS();

    nrfx_spis_config_t spis_config = NRFX_SPIS_DEFAULT_CONFIG(SCK_PIN_SLAVE,
                                                              MOSI_PIN_SLAVE,
                                                              MISO_PIN_SLAVE,
                                                              CSN_PIN_SLAVE);

    status = nrfx_spis_init(&spis_inst, &spis_config, spis_handler, p_context);
    NRFX_ASSERT(status == NRFX_SUCCESS);

#if defined(__ZEPHYR__)
    IRQ_DIRECT_CONNECT(NRFX_IRQ_NUMBER_GET(NRF_SPIS_INST_GET(SPIS_INST_IDX)), IRQ_PRIO_LOWEST,
                       NRFX_SPIS_INST_HANDLER_GET(SPIS_INST_IDX), 0);
#endif

    status = nrfx_spis_buffers_set(&spis_inst,
                                   m_tx_buffer_slave, sizeof(m_tx_buffer_slave),
                                   m_rx_buffer_slave, sizeof(m_rx_buffer_slave));
    NRFX_ASSERT(status == NRFX_SUCCESS);

    while (1)
    {
        NRFX_EXAMPLE_LOG_PROCESS();
    }
}

/** @} */
